// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library passive_data.test.generate_code_test;

import 'package:test/test.dart';
import 'package:passive_data/passive_data.dart';

/// 'd' is used as data access name.
const Map<String, String> typeToPds = const {
  'String': 'd',
  'List<Point>': 'toPdsList(d)',
  'Set<Point>': 'toPdsList(d.toList())',
  'Map<int, Point>': 'toPdsMap(d)',
  'Point': 'd.toPds()'
};

/// 'd' is used as data access name. Cast is set to false initially.
/// All spaces and line endings are stripped.
Map<List<String>, String> pdsToType = {
  ['dynamic']: 'd',
  ['String']: 'd',
  ['Set<String>', 'List<String>']: 'new Set<String>.from(d)',
  ['List<Point>', 'List<String>']: '''
new List<Point>.generate(d.length, (int i1) => new Point.fromPds(d[i1] as String))
''',
  ['Map<String, Map<int, dynamic>>']: '''
new Map<String, Map<int, dynamic>>.fromIterable(d.keys,
    key: (String key1) => key1,
    value: (String key1) => new Map<int, dynamic>.from(d[key1]))
''',
  ['Map<String, Point>', 'Map<String, Map<int, dynamic>>']: '''
new Map<String, Point>.fromIterable(d.keys,
    key: (String key1) => key1,
    value: (String key1) => new Point.fromPds(new Map<int, dynamic>.from(d[key1])))
''',
  ['Map<int, Point>', 'Map<int, Map<int, dynamic>>']: '''
new Map<int, Point>.fromIterable(d.keys,
  key: (int key1) => key1,
  value: (int key1) => new Point.fromPds(new Map<int, dynamic>.from(d[key1])))
''',
  ['Point', 'Map<String, dynamic>']:
      'new Point.fromPds(new Map<String, dynamic>.from(d))',
  ['List<Set<MyObject>>', 'List<List<int>>']: '''
new List<Set<MyObject>>.generate(d.length,
    (int i1) => new List<MyObject>.generate(d[i1].length,
        (int i2) => new MyObject.fromPds(d[i1][i2] as int)).toSet())
'''
};

/// All throw an ArgumentError.
List<List<String>> pdsToTypeExceptions = [
  ['Map<int, Point>', 'Set<int>'],
  ['Map<int, Point>', ''],
  ['List<Point>', 'int'],
  ['List<Point>', '']
];

void main() {
  test('Test getTypeToPdsCode', () {
    typeToPds.forEach((String type, String expected) {
      expect(_stripSpacesAndLf(getTypeToPdsCode(type, 'd')),
          equals(_stripSpacesAndLf(expected)));
    });
  });

  test('Test getPdsToTypeCode', () {
    pdsToType.forEach((List<String> types, String expected) {
      expect(
          _stripSpacesAndLf(getPdsToTypeCode(
              types[0], 'd', types.length > 1 ? types[1] : '', false)),
          equals(_stripSpacesAndLf(expected)));
    });
  });

  test('Test getPdsToTypeCode exceptions', () {
    pdsToTypeExceptions.forEach((List<String> types) {
      expect(() => getPdsToTypeCode(types[0], 'd', types[1], false),
          throwsArgumentError);
    });
  });
}

/// Remove spaces and line feeds from the given string.
String _stripSpacesAndLf(String str) {
  str = str.replaceAll(' ', '');
  str = str.replaceAll('\n', '');
  return str;
}
